<?php

namespace Drupal\details_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\text\Plugin\Field\FieldFormatter\TextDefaultFormatter;

/**
 * Plugin implementation of the 'details_field' formatter.
 */
#[FieldFormatter(
  id: 'details_field',
  label: new TranslatableMarkup('Details element'),
  field_types: [
    'details_field',
  ],
)]
class DetailsFieldFormatter extends TextDefaultFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'automatic_id' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['automatic_id'] = [
      '#title' => $this->t('Automatically generated ID'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('automatic_id'),
      '#description' => $this->t('Automatically generates an ID for each details element, with a URL-friendly value based on the details summary. This value will be overridden if the attribute ID is allowed and exposed for editors to set it manually.'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if ($this->getSetting('automatic_id')) {
      $summary[] = $this->t('Automatically generated ID enabled');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    // The ProcessedText element already handles cache context & tag bubbling.
    // @see \Drupal\filter\Element\ProcessedText::preRenderText()
    foreach ($items as $delta => $item) {
      $attributes = [];
      if ($this->getSetting('automatic_id')) {
        // Generate URL-friendly ID based on the details item summary.
        // Make text lowercase, remove all characters except alphanumeric,
        // trim text, and replace spaces with hyphens.
        $attributes['id'] = str_replace(' ', '-', trim(preg_replace('/[^A-Za-z0-9 ]/', '', strtolower($item->summary))));
      }
      if (!empty($name = $item->name)) {
        $attributes['name'] = $name;
      }
      if (!empty($attrs = $this->arrayFilterRecursive($item->attributes))) {
        $attributes = array_merge($attributes, $attrs);
      }

      $elements[$delta] = [
        '#type' => 'details',
        '#title' => $item->summary,
        '#open' => (bool) $item->open,
        '#attributes' => $attributes,
      ];
      $elements[$delta]['content'] = [
        '#type' => 'processed_text',
        '#text' => $item->value,
        '#format' => $item->format,
        '#langcode' => $item->getLangcode(),
      ];
    }

    return $elements;
  }

  /**
   * Removes empty items from a multidimensional array recursively.
   *
   * @param array|null $array
   *   Array to be filtered.
   *
   * @return array
   *   Array containing attribute options.
   */
  private function arrayFilterRecursive(?array $array): array {
    foreach ($array as $key => $value) {
      if (is_array($value)) {
        $array[$key] = $this->arrayFilterRecursive(array_filter($value));
      }
    }
    return array_filter($array);
  }

}
