<?php

namespace Drupal\details_field\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\MapDataDefinition;
use Drupal\text\Plugin\Field\FieldType\TextFieldItemList;
use Drupal\text\Plugin\Field\FieldType\TextWithSummaryItem;

/**
 * Plugin implementation of the 'details_field' field type.
 */
#[FieldType(
  id: "details_field",
  label: new TranslatableMarkup("Details element"),
  description: [
    new TranslatableMarkup("Creates a disclosure widget in which information is usually visible upon user interaction"),
    new TranslatableMarkup("By default, the widget is closed. When open, it expands, and displays the content within"),
    new TranslatableMarkup("Ideal for longer texts, like body or description"),
    new TranslatableMarkup("A summary is required for toggling the interactive widget"),
    new TranslatableMarkup("May use more storage and be slower for searching and sorting"),
  ],
  category: "formatted_text",
  weight: 1,
  default_widget: "details_field",
  default_formatter: "details_field",
  list_class: TextFieldItemList::class,
)]
class DetailsFieldItem extends TextWithSummaryItem {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);
    $schema['columns']['open'] = [
      'description' => 'Defines if the details element should be open by default.',
      'type' => 'int',
      'size' => 'tiny',
      'default' => 0,
    ];
    $schema['columns']['name'] = [
      'description' => 'Specifies the name attribute value for the details element.',
      'type' => 'varchar',
      'length' => 255,
      'default' => '',
    ];
    $schema['columns']['attributes'] = [
      'description' => 'Serialized array of extra attributes for the details element.',
      'type' => 'blob',
      'size' => 'big',
      'serialize' => TRUE,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);

    $properties['open'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Open'));
    $properties['name'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Name'));
    $properties['attributes'] = MapDataDefinition::create()
      ->setLabel(new TranslatableMarkup('Attributes'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    if (isset($values) && !is_array($values)) {
      $values = [static::mainPropertyName() => $values];
    }
    if (isset($values)) {
      $values += [
        'attributes' => [],
      ];
    }
    if (is_string($values['attributes'])) {
      $values['attributes'] = unserialize($values['attributes'], ['allowed_classes' => ['stdClass']]);
    }
    parent::setValue($values, $notify);
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::fieldSettingsForm($form, $form_state);

    // Disable summary related fields from the settings form since they should
    // always be enabled.
    $element['display_summary']['#access'] = FALSE;
    $element['required_summary']['#access'] = FALSE;

    return $element;
  }

}
