<?php

namespace Drupal\details_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\details_field\DetailsFieldManager;
use Drupal\text\Plugin\Field\FieldWidget\TextareaWithSummaryWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'details_field' widget.
 *
 * @phpstan-consistent-constructor
 */
#[FieldWidget(
  id: 'details_field',
  label: new TranslatableMarkup('Details element'),
  field_types: ['details_field'],
)]
class DetailsFieldWidget extends TextareaWithSummaryWidget {

  /**
   * The details field manager.
   *
   * @var \Drupal\details_field\DetailsFieldManager
   */
  protected $detailsFieldManager;

  /**
   * Constructs the Details Field widget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\details_field\DetailsFieldManager $details_field_manager
   *   The details field manager.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    DetailsFieldManager $details_field_manager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->detailsFieldManager = $details_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.details_field'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $default_settings = parent::defaultSettings();

    $default_settings['summary_rows'] = '1';
    $default_settings['allowed_attributes'] = [
      'open' => TRUE,
      'name' => TRUE,
      'id' => FALSE,
      'class' => FALSE,
      'aria-label' => FALSE,
    ];
    return $default_settings;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    // Remove this form item since the summary should always be visible.
    unset($element['show_summary']);

    $allowed_attributes = $this->getSetting('allowed_attributes');
    $allowed_attributes = !empty($allowed_attributes) ? array_keys(array_filter($allowed_attributes)) : [];

    $element['allowed_attributes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed attributes'),
      '#options' => $this->detailsFieldManager->getOptionsFromDefinitions(),
      '#default_value' => array_combine($allowed_attributes, $allowed_attributes),
      '#description' => $this->t('Select the attributes to allow editors to set.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $allowed_attributes = array_filter($this->getSetting('allowed_attributes'));
    if (!empty($allowed_attributes)) {
      $attribute_options = $this->detailsFieldManager->getOptionsFromDefinitions();
      foreach (array_keys($allowed_attributes) as $key) {
        $allowed_attributes[$key] = $attribute_options[$key];
      }

      $summary[] = $this->t('Allowed attributes: @allowed_attributes', ['@allowed_attributes' => implode(', ', array_values($allowed_attributes))]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    /** @var \Drupal\details_field\Plugin\Field\FieldType\DetailsFieldItem $item */
    $item = $items[$delta];

    $element['summary'] = [
      '#type' => 'textarea',
      '#rows' => $this->getSetting('summary_rows'),
      '#default_value' => $item->get('summary')->getValue(),
      '#title' => $this->t('Summary'),
      '#description' => $this->t('The text of the interactive heading for the details element.'),
      '#weight' => -10,
      '#required' => FALSE,
    ];

    $element['extra_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Extra Settings'),
      '#weight' => 10,
    ];

    // Build Attributes widget.
    $allowed_attributes = $this->getSetting('allowed_attributes');
    if (!empty($allowed_attributes)) {
      $attribute_definitions = $this->detailsFieldManager->getDefinitions();
      $attribute_values = $item->get('attributes')->getValue();
      $attribute_values['class'] = isset($attribute_values['class']) ? implode(' ', $attribute_values['class']) : NULL;

      foreach ($attribute_definitions as $key => $definition) {
        if ($allowed_attributes[$key]) {
          $form_field = [
            '#type' => $definition['type'],
            '#title' => $definition['title'],
            '#description' => $definition['description'],
          ];

          if ($definition['specific_field']) {
            $form_field['#default_value'] = $items[$delta]->{$key};
            $element['extra_settings'][$key] = $form_field;
          }
          else {
            $form_field['#default_value'] = $attribute_values[$key] ?? NULL;
            $element['extra_settings']['attributes'][$key] = $form_field;
          }
        }
      }

      if (isset($element['extra_settings']['attributes'])) {
        $element['extra_settings']['attributes']['#type'] = 'fieldset';
        $element['extra_settings']['attributes']['#title'] = $this->t('Attributes');
      }
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as &$item) {
      if (isset($item['extra_settings'])) {
        foreach ($item['extra_settings'] as $key => $value) {
          $item[$key] = $value;
        }
      }
      // Serialize attribute values to save them into the database.
      if (isset($item['attributes'])) {
        if (isset($item['attributes']['class'])) {
          // Handle multiple classes passed separated by space.
          $item['attributes']['class'] = explode(' ', $item['attributes']['class']);
        }
        $item['attributes'] = serialize($item['attributes']);
      }
    }
    return $values;
  }

}
